# -*- encoding: utf-8 -*-

from django.db import models
from models import Parent, PermissionMeta, Region, Adresse, Personne
from implantation import TypeImplantation, Implantation
from django_historique.models import History
from django.db.models.signals import pre_save, pre_delete
from django.conf import settings


EE_STATUT_CHOICES = (('N', 'Non membre'), \
                     ('A', 'Associé'), \
                     ('T', 'Titulaire'), \
                     ('C', 'Candidat'))

E_QUALITE_CHOICES = (('ESR', 'Etablissement d\'enseignement superieur et de recherche'), \
                     ('CIR', 'Centre ou institution de recherche'), \
                     ('RES', 'Reseaux'))


## Etablissement
class EtablissementGeneral (Parent):
    __metaclass__ = PermissionMeta

    nom = models.CharField(max_length=255,
                           verbose_name='Intitulé (français)')
    nom_local = models.CharField(max_length=255,
                                null = True, blank = True,
                                verbose_name = 'Intitulé (langue d\'origine)')
    region = models.ForeignKey(Region,
                               null = True,
                               verbose_name = 'Region de rattachement')
    tel1 = models.CharField(max_length=32, null = True,
                            verbose_name = 'Telephone 1')
    poste1 = models.CharField(max_length=16, null = True,
                              verbose_name = 'Poste', blank = True)
    tel2 = models.CharField(max_length=32, null = True,
                            verbose_name = 'Telephone 2', blank = True)
    poste2 = models.CharField(max_length=16, null = True,
                              verbose_name = 'Poste', blank = True)
    fax = models.CharField(max_length=32, null = True, blank = True)
    fax2 = models.CharField(max_length=32, null = True, blank = True,
                            verbose_name = "Fax 2")
    courriel = models.EmailField(max_length=128, null = True, blank = True)
    url = models.CharField(max_length=128,
                           null = True, verbose_name = 'Site internet',
                           blank = True)
    historique = models.TextField(null = True, verbose_name = 'Historique RG',
                                  blank = True)
    etudiants = models.IntegerField(null = True,
                                    verbose_name = 'Nombre d\'étudiants',
                                    blank = True)
    an_ref_etudiants = models.IntegerField(null = True,
                                           verbose_name = 'Année de référence (Étudiants)',
                                           blank = True)
    enseignants = models.IntegerField(null = True,
                                      verbose_name = 'Nombre d\'enseignants',
                                      blank = True)
    an_ref_enseignants = models.IntegerField(null = True,
                                             verbose_name = 'Année de référence (Enseignants)',
                                             blank = True)
    chercheurs = models.IntegerField(null = True,
                                     verbose_name = 'Nombre de chercheurs',
                                     blank = True)
    an_ref_chercheurs = models.IntegerField(null = True,
                                            verbose_name = 'Année de référence (Chercheurs)',
                                            blank = True)
    mise_a_jour = models.DateField(db_column = 'datemaj', auto_now = True,
                                 verbose_name = "Date de mise à jour")
    actif = models.NullBooleanField()
    nom_long = models.CharField(max_length=36,
                                verbose_name = 'Nom long (Coda)')
    nom_court = models.CharField(max_length=20,
                                 verbose_name = 'Nom court (Coda)')
    historique_annuaire = models.TextField(null = True,
                                           verbose_name = 'Historique annuaire', 
                                           blank = True)
    descriptif = models.TextField(null = True, blank = True)
    commentaire = models.TextField(null = True, verbose_name = 'Commentaires',
                                   blank = True)
    qualite = models.CharField(max_length = 3, null = True,
                               verbose_name = u'Qualité de l\'établissement',
                               choices = E_QUALITE_CHOICES, blank = True)
    prive = models.NullBooleanField(verbose_name = 'Privé')
    adhesion = models.IntegerField(null = True,
                                   verbose_name = 'Année d\'adhesion',
                                   blank = True)
    implantation = models.ForeignKey(Implantation,
                                     db_column = 'implantation', null = True,
                                     verbose_name = 'Implantation de rattachement',
                                     blank = True)
    sigle = models.CharField(max_length=16, null = True, blank = True)
    statut = models.CharField(max_length = 1, null = True,
                              choices = EE_STATUT_CHOICES, blank = True)
    statut_effectif = models.DateField(null = True,
                                       verbose_name = "Effectif le")
    commentaire_statut = models.TextField(null = True,
                                         verbose_name = 'Commentaires statut',
                                         blank = True)

    class Meta:
        abstract = True

class Etablissement (EtablissementGeneral):
    id = models.AutoField(primary_key=True)
    responsables = models.ManyToManyField(Personne, blank=True, null=True,
                                          through='ResponsableEtablissement')
    adresse = models.OneToOneField (Adresse, blank=True, null=True)
    def __unicode__ (self):
        return self.nom
    class Meta:
        #db_table = u'etablissement'
        ordering = ["nom"]

class EtablissementHistory(EtablissementGeneral, History):
    class Meta:
        #db_table = u'histo_etablissement'
        pass


class ResponsableEtablissement(Parent):
    __metaclass__ = PermissionMeta
    etablissement = models.ForeignKey(Etablissement)
    personne = models.ForeignKey(Personne)
    priorite = models.CharField(max_length=3, verbose_name = 'Priorité', 
                                blank=True, null=True)

    def __unicode__ (self):
        return "Responsable: %s, %s" % (self.personne, self.etablissement)


